@@ -7,6 +7,7 @@
  * Copyright 1999      Alex Korobka
  * Copyright 2003      Thomas Mertes
  * Crc32 code Copyright 1986 Gary S. Brown (Public domain)
+ * Copyright 2025      Zhiyi Zhang for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,7 +29,9 @@
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
+#include "winbase.h"
 #include "winternl.h"
+#include "ntuser.h"
 #include "wine/debug.h"
 #include "wine/exception.h"
 #include "ntdll_misc.h"
@@ -167,6 +170,399 @@ NTSTATUS WINAPI vDbgPrintExWithPrefix( LPCSTR prefix, ULONG id, ULONG level, LPC
     return STATUS_SUCCESS;
 }
 
+
+static struct ntuser_client_procs_table user_procs;
+
+#define DEFINE_USER_FUNC(name,ptr) \
+    LRESULT WINAPI name( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp ) { return (ptr)( hwnd, msg, wp, lp ); }
+
+#define USER_FUNC(name,proc) \
+    DEFINE_USER_FUNC( Ntdll##name##_A, user_procs.A[proc][0] ) \
+    DEFINE_USER_FUNC( Ntdll##name##_W, user_procs.W[proc][0] )
+ALL_NTUSER_CLIENT_PROCS
+#undef USER_FUNC
+
+static const struct ntuser_client_procs_table user_funcs =
+{
+#define USER_FUNC(name,proc) .A[proc] = { Ntdll##name##_A }, .W[proc] = { Ntdll##name##_W },
+    ALL_NTUSER_CLIENT_PROCS
+#undef USER_FUNC
+};
+
+static BOOL user_procs_init_done;
+
+/***********************************************************************
+ *	     RtlInitializeNtUserPfn   (NTDLL.@)
+ */
+NTSTATUS WINAPI RtlInitializeNtUserPfn( const void *client_procsA, ULONG procsA_size,
+                                        const void *client_procsW, ULONG procsW_size,
+                                        const void *client_workers, ULONG workers_size )
+{
+    if (procsA_size % sizeof(ntuser_client_func_ptr) || procsA_size > sizeof(user_procs.A) ||
+        procsW_size % sizeof(ntuser_client_func_ptr) || procsW_size > sizeof(user_procs.W) ||
+        workers_size % sizeof(ntuser_client_func_ptr) || workers_size > sizeof(user_procs.workers))
+        return STATUS_INVALID_PARAMETER;
+
+    if (user_procs_init_done) return STATUS_INVALID_PARAMETER;
+    memcpy( user_procs.A, client_procsA, procsA_size );
+    memcpy( user_procs.W, client_procsW, procsW_size );
+    memcpy( user_procs.workers, client_workers, workers_size );
+    user_procs_init_done = TRUE;
+    return STATUS_SUCCESS;
+}
+
+/***********************************************************************
+ *	     RtlRetrieveNtUserPfn   (NTDLL.@)
+ */
+NTSTATUS WINAPI RtlRetrieveNtUserPfn( const void **client_procsA,
+                                      const void **client_procsW,
+                                      const void **client_workers )
+{
+    if (!user_procs_init_done) return STATUS_INVALID_PARAMETER;
+    *client_procsA  = user_funcs.A;
+    *client_procsW  = user_funcs.W;
+    *client_workers = user_funcs.workers;
+    return STATUS_SUCCESS;
+}
+
+/***********************************************************************
+ *	     RtlResetNtUserPfn   (NTDLL.@)
+ */
+NTSTATUS WINAPI RtlResetNtUserPfn(void)
+{
+    if (!user_procs_init_done) return STATUS_INVALID_PARAMETER;
+    user_procs_init_done = FALSE;
+    memset( &user_procs, 0, sizeof(user_procs) );
+    return STATUS_SUCCESS;
+}
+
+/* data is a place holder to align stored data on a 8 byte boundary */
+struct rtl_generic_table_entry
+{
+    RTL_SPLAY_LINKS splay_links;
+    LIST_ENTRY list_entry;
+    LONGLONG data;
+};
+
+static void *get_data_from_splay_links(RTL_SPLAY_LINKS *links)
+{
+    return (unsigned char *)links + FIELD_OFFSET(struct rtl_generic_table_entry, data);
+}
+
+static LIST_ENTRY *get_list_entry_from_splay_links(RTL_SPLAY_LINKS *links)
+{
+    return (LIST_ENTRY *)((unsigned char *)links + FIELD_OFFSET(struct rtl_generic_table_entry, list_entry));
+}
+
+static RTL_SPLAY_LINKS *rtl_splay_find(RTL_GENERIC_TABLE *table, void *value)
+{
+    RTL_GENERIC_COMPARE_RESULTS result;
+    RTL_SPLAY_LINKS *child;
+
+    child = table->TableRoot;
+    while (child)
+    {
+        result = table->CompareRoutine(table, get_data_from_splay_links(child), value);
+        if (result == GenericLessThan)
+            child = child->RightChild;
+        else if (result == GenericGreaterThan)
+            child = child->LeftChild;
+        else
+            return child;
+    }
+    return NULL;
+}
+
+static void rtl_splay_replace(RTL_SPLAY_LINKS *x, RTL_SPLAY_LINKS *y, RTL_SPLAY_LINKS **root)
+{
+    if (RtlIsRoot(x))
+        *root = y;
+    else if (RtlIsLeftChild(x))
+        x->Parent->LeftChild = y;
+    else
+        x->Parent->RightChild = y;
+
+    if (y)
+        y->Parent = RtlIsRoot(x) ? y : x->Parent;
+}
+
+static void rtl_splay_left_rotate(RTL_SPLAY_LINKS *x)
+{
+    RTL_SPLAY_LINKS *y = x->RightChild;
+
+    if (y)
+    {
+        x->RightChild = y->LeftChild;
+        if (y->LeftChild)
+            y->LeftChild->Parent = x;
+
+        y->Parent = RtlIsRoot(x) ? y : x->Parent;
+    }
+
+    if (!RtlIsRoot(x))
+    {
+        if (RtlIsLeftChild(x))
+            x->Parent->LeftChild = y;
+        else
+            x->Parent->RightChild = y;
+    }
+
+    if (y)
+        y->LeftChild = x;
+    x->Parent = y;
+}
+
+static void rtl_splay_right_rotate(RTL_SPLAY_LINKS *x)
+{
+    RTL_SPLAY_LINKS *y = x->LeftChild;
+
+    if (y)
+    {
+        x->LeftChild = y->RightChild;
+        if (y->RightChild)
+            y->RightChild->Parent = x;
+
+        y->Parent = RtlIsRoot(x) ? y : x->Parent;
+    }
+
+    if (!RtlIsRoot(x))
+    {
+        if (RtlIsLeftChild(x))
+            x->Parent->LeftChild = y;
+        else
+            x->Parent->RightChild = y;
+    }
+
+    if (y)
+        y->RightChild = x;
+    x->Parent = y;
+}
+
+/******************************************************************************
+ *  RtlSubtreePredecessor           [NTDLL.@]
+ */
+RTL_SPLAY_LINKS * WINAPI RtlSubtreePredecessor(RTL_SPLAY_LINKS *links)
+{
+    RTL_SPLAY_LINKS *child;
+
+    TRACE("(%p)\n", links);
+
+    child = RtlLeftChild(links);
+    if (!child)
+        return NULL;
+
+    while (RtlRightChild(child))
+        child = RtlRightChild(child);
+
+    return child;
+}
+
+/******************************************************************************
+ *  RtlSubtreeSuccessor           [NTDLL.@]
+ */
+RTL_SPLAY_LINKS * WINAPI RtlSubtreeSuccessor(RTL_SPLAY_LINKS *links)
+{
+    RTL_SPLAY_LINKS *child;
+
+    TRACE("(%p)\n", links);
+
+    child = RtlRightChild(links);
+    if (!child)
+        return NULL;
+
+    while (RtlLeftChild(child))
+        child = RtlLeftChild(child);
+
+    return child;
+}
+
+/******************************************************************************
+ *  RtlRealPredecessor           [NTDLL.@]
+ */
+RTL_SPLAY_LINKS * WINAPI RtlRealPredecessor(RTL_SPLAY_LINKS *links)
+{
+    PRTL_SPLAY_LINKS child;
+
+    TRACE("(%p)\n", links);
+
+    child = RtlLeftChild(links);
+    if (child)
+    {
+        while (RtlRightChild(child))
+            child = RtlRightChild(child);
+        return child;
+    }
+
+    child = links;
+    while (RtlIsLeftChild(child))
+        child = RtlParent(child);
+
+    if (RtlIsRightChild(child))
+        return RtlParent(child);
+
+    return NULL;
+}
+
+/******************************************************************************
+ *  RtlRealSuccessor           [NTDLL.@]
+ */
+RTL_SPLAY_LINKS * WINAPI RtlRealSuccessor(RTL_SPLAY_LINKS *links)
+{
+    PRTL_SPLAY_LINKS child;
+
+    TRACE("(%p)\n", links);
+
+    child = RtlRightChild(links);
+    if (child)
+    {
+        while (RtlLeftChild(child))
+            child = RtlLeftChild(child);
+        return child;
+    }
+
+    child = links;
+    while (RtlIsRightChild(child))
+        child = RtlParent(child);
+
+    if (RtlIsLeftChild(child))
+        return RtlParent(child);
+
+    return NULL;
+}
+
+/******************************************************************************
+ *  RtlSplay           [NTDLL.@]
+ */
+RTL_SPLAY_LINKS * WINAPI RtlSplay(RTL_SPLAY_LINKS *links)
+{
+    TRACE("(%p)\n", links);
+
+    while (!RtlIsRoot(links))
+    {
+        if (RtlIsRoot(links->Parent))
+        {
+            /* Zig */
+            if (RtlIsLeftChild(links))
+                rtl_splay_right_rotate(links->Parent);
+            /* Zag */
+            else
+                rtl_splay_left_rotate(links->Parent);
+        }
+        /* Zig-Zig */
+        else if (RtlIsLeftChild(links->Parent) && RtlIsLeftChild(links))
+        {
+            rtl_splay_right_rotate(links->Parent->Parent);
+            rtl_splay_right_rotate(links->Parent);
+        }
+        /* Zag-Zag */
+        else if (RtlIsRightChild(links->Parent) && RtlIsRightChild(links))
+        {
+            rtl_splay_left_rotate(links->Parent->Parent);
+            rtl_splay_left_rotate(links->Parent);
+        }
+        /* Zig-Zag */
+        else if (RtlIsRightChild(links->Parent) && RtlIsLeftChild(links))
+        {
+            rtl_splay_right_rotate(links->Parent);
+            rtl_splay_left_rotate(links->Parent);
+        }
+        /* Zag-Zig */
+        else
+        {
+            rtl_splay_left_rotate(links->Parent);
+            rtl_splay_right_rotate(links->Parent);
+        }
+    }
+
+    return links;
+}
+
+/******************************************************************************
+ *  RtlDeleteNoSplay           [NTDLL.@]
+ */
+void WINAPI RtlDeleteNoSplay(RTL_SPLAY_LINKS *links, RTL_SPLAY_LINKS **root)
+{
+    TRACE("(%p, %p)\n", links, root);
+
+    if (RtlIsRoot(links) && !RtlLeftChild(links) && !RtlRightChild(links))
+    {
+        *root = NULL;
+    }
+    else  if (!links->LeftChild)
+    {
+        rtl_splay_replace(links, links->RightChild, root);
+    }
+    else if (!links->RightChild)
+    {
+        rtl_splay_replace(links, links->LeftChild, root);
+    }
+    else
+    {
+        RTL_SPLAY_LINKS *predecessor = RtlSubtreePredecessor(links);
+        if (predecessor->Parent != links)
+        {
+            rtl_splay_replace(predecessor, predecessor->LeftChild, root);
+            RtlInsertAsLeftChild(predecessor, links->LeftChild);
+        }
+        rtl_splay_replace(links, predecessor, root);
+        RtlInsertAsRightChild(predecessor, links->RightChild);
+    }
+}
+
+/******************************************************************************
+ *  RtlDelete           [NTDLL.@]
+ */
+RTL_SPLAY_LINKS * WINAPI RtlDelete(RTL_SPLAY_LINKS *links)
+{
+    RTL_SPLAY_LINKS *root, *to_splay;
+
+    TRACE("(%p)\n", links);
+
+    if (RtlIsRoot(links) && !RtlLeftChild(links) && !RtlRightChild(links))
+    {
+        return NULL;
+    }
+    else if (!links->LeftChild)
+    {
+        rtl_splay_replace(links, links->RightChild, &root);
+        if (RtlIsRoot(links))
+            return links->RightChild;
+
+        to_splay = links->Parent;
+    }
+    else if (!links->RightChild)
+    {
+        rtl_splay_replace(links, links->LeftChild, &root);
+        if (RtlIsRoot(links))
+            return links->LeftChild;
+
+        to_splay = links->Parent;
+    }
+    else
+    {
+        RTL_SPLAY_LINKS *predecessor = RtlSubtreePredecessor(links);
+        if (predecessor->Parent != links)
+        {
+            rtl_splay_replace(predecessor, predecessor->LeftChild, &root);
+            RtlInsertAsLeftChild(predecessor, links->LeftChild);
+            /* Delete operation first swap the value of node to delete and that of the predecessor
+             * and then delete the predecessor instead. Finally, the parent of the actual deleted
+             * node, which is the predecessor, is splayed afterwards */
+            to_splay = predecessor->Parent;
+        }
+        else
+        {
+            /* links is the parent of predecessor. So after swapping, the parent of links is in
+             * fact the predecessor. So predecessor gets splayed */
+            to_splay = predecessor;
+        }
+        rtl_splay_replace(links, predecessor, &root);
+        RtlInsertAsRightChild(predecessor, links->RightChild);
+    }
+
+    return RtlSplay(to_splay);
+}
+
 /******************************************************************************
  *  RtlInitializeGenericTable           [NTDLL.@]
  */
@@ -188,6 +584,91 @@ void WINAPI RtlInitializeGenericTable(RTL_GENERIC_TABLE *table, PRTL_GENERIC_COM
     table->TableContext = context;
 }
 
+/******************************************************************************
+ *  RtlIsGenericTableEmpty           [NTDLL.@]
+ */
+BOOLEAN WINAPI RtlIsGenericTableEmpty(RTL_GENERIC_TABLE *table)
+{
+    TRACE("(%p)\n", table);
+    return !table->TableRoot;
+}
+
+/***********************************************************************
+ *           RtlInsertElementGenericTable  (NTDLL.@)
+ */
+void * WINAPI RtlInsertElementGenericTable(RTL_GENERIC_TABLE *table, void *value, CLONG size, BOOLEAN *new_element)
+{
+    RTL_SPLAY_LINKS *child, *parent = NULL;
+    RTL_GENERIC_COMPARE_RESULTS result;
+    void *buffer;
+
+    TRACE("(%p, %p, %lu, %p)\n", table, value, size, new_element);
+
+    child = table->TableRoot;
+    while (child)
+    {
+        buffer = get_data_from_splay_links(child);
+        result = table->CompareRoutine(table, buffer, value);
+        parent = child;
+        if (result == GenericLessThan)
+        {
+            child = child->RightChild;
+        }
+        else if (result == GenericGreaterThan)
+        {
+            child = child->LeftChild;
+        }
+        else
+        {
+            if (new_element)
+                *new_element = FALSE;
+            return buffer;
+        }
+    }
+
+    /* data should be stored on a 8 byte boundary */
+    child = (RTL_SPLAY_LINKS *)table->AllocateRoutine(table, size + FIELD_OFFSET(struct rtl_generic_table_entry, data));
+    RtlInitializeSplayLinks(child);
+    InsertTailList(&table->InsertOrderList, get_list_entry_from_splay_links(child));
+    buffer = get_data_from_splay_links(child);
+    memcpy(buffer, value, size);
+
+    if (parent)
+    {
+        buffer = get_data_from_splay_links(parent);
+        result = table->CompareRoutine(table, buffer, value);
+        if (result == GenericLessThan)
+            RtlInsertAsRightChild(parent, child);
+        else
+            RtlInsertAsLeftChild(parent, child);
+    }
+
+    if (new_element)
+        *new_element = TRUE;
+    table->TableRoot = RtlSplay(child);
+    table->NumberGenericTableElements++;
+    return get_data_from_splay_links(child);
+}
+
+BOOLEAN WINAPI RtlDeleteElementGenericTable(RTL_GENERIC_TABLE *table, void *value)
+{
+    RTL_SPLAY_LINKS *child;
+
+    TRACE("(%p, %p)\n", table, value);
+
+    child = rtl_splay_find(table, value);
+    if (!child)
+        return FALSE;
+
+    table->TableRoot = RtlDelete(child);
+    RemoveEntryList(get_list_entry_from_splay_links(child));
+    table->NumberGenericTableElements--;
+    table->OrderedPointer = &table->InsertOrderList;
+    table->WhichOrderedElement = 0;
+    table->FreeRoutine(table, child);
+    return TRUE;
+}
+
 /******************************************************************************
  *  RtlEnumerateGenericTableWithoutSplaying           [NTDLL.@]
  */
@@ -221,10 +702,18 @@ void * WINAPI RtlGetElementGenericTable(RTL_GENERIC_TABLE *table, ULONG index)
 /******************************************************************************
  *  RtlLookupElementGenericTable           [NTDLL.@]
  */
-void * WINAPI RtlLookupElementGenericTable(RTL_GENERIC_TABLE *table, void *buffer)
+void * WINAPI RtlLookupElementGenericTable(RTL_GENERIC_TABLE *table, void *value)
 {
-    FIXME("(%p, %p) stub!\n", table, buffer);
-    return NULL;
+    RTL_SPLAY_LINKS *child;
+
+    TRACE("(%p, %p)\n", table, value);
+
+    child = rtl_splay_find(table, value);
+    if (!child)
+        return FALSE;
+
+    table->TableRoot = RtlSplay(child);
+    return get_data_from_splay_links(child);
 }
 
 /******************************************************************************
