@@ -18,10 +18,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "user_private.h"
 #include "controls.h"
 #include "winver.h"
-#include "wine/server.h"
 #include "wine/asm.h"
 #include "wine/exception.h"
 #include "wine/debug.h"
@@ -29,51 +30,57 @@
 WINE_DEFAULT_DEBUG_CHANNEL(win);
 
 
-/*******************************************************************
- *           list_window_children
- *
- * Build an array of the children of a given window. The array must be
- * freed with HeapFree. Returns NULL when no windows are found.
+#ifdef __i386__
+/* Some apps pass a non-stdcall proc to EnumChildWindows,
+ * so we need a small assembly wrapper to call the proc.
  */
-static HWND *list_window_children( HDESK desktop, HWND hwnd, UNICODE_STRING *class, DWORD tid )
+extern LRESULT enum_callback_wrapper( WNDENUMPROC proc, HWND hwnd, LPARAM lparam );
+__ASM_GLOBAL_FUNC( enum_callback_wrapper,
+    "pushl %ebp\n\t"
+    __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
+    __ASM_CFI(".cfi_rel_offset %ebp,0\n\t")
+    "movl %esp,%ebp\n\t"
+    __ASM_CFI(".cfi_def_cfa_register %ebp\n\t")
+    "pushl 16(%ebp)\n\t"
+    "pushl 12(%ebp)\n\t"
+    "call *8(%ebp)\n\t"
+    "leave\n\t"
+    __ASM_CFI(".cfi_def_cfa %esp,4\n\t")
+    __ASM_CFI(".cfi_same_value %ebp\n\t")
+    "ret" )
+#else
+static inline LRESULT enum_callback_wrapper( WNDENUMPROC proc, HWND hwnd, LPARAM lparam )
+{
+    return proc( hwnd, lparam );
+}
+#endif /* __i386__ */
+
+/*******************************************************************
+ *           enum_windows
+ */
+static BOOL enum_windows( HDESK desktop, HWND hwnd, DWORD tid, BOOL children,
+                          WNDENUMPROC proc, LPARAM param )
 {
     HWND *list;
-    int i, size = 128;
-    ATOM atom = class ? get_int_atom_value( class ) : 0;
-
-    /* empty class is not the same as NULL class */
-    if (!atom && class && !class->Length) return NULL;
+    ULONG i, size = 128;
+    BOOL ret = !children;  /* EnumChildWindows returns FALSE on empty list, the others TRUE */
+    NTSTATUS status;
 
     for (;;)
     {
-        int count = 0;
-
-        if (!(list = HeapAlloc( GetProcessHeap(), 0, size * sizeof(HWND) ))) break;
-
-        SERVER_START_REQ( get_window_children )
-        {
-            req->desktop = wine_server_obj_handle( desktop );
-            req->parent = wine_server_user_handle( hwnd );
-            req->tid = tid;
-            req->atom = atom;
-            if (!atom && class) wine_server_add_data( req, class->Buffer, class->Length );
-            wine_server_set_reply( req, list, (size-1) * sizeof(user_handle_t) );
-            if (!wine_server_call( req )) count = reply->count;
-        }
-        SERVER_END_REQ;
-        if (count && count < size)
-        {
-            /* start from the end since HWND is potentially larger than user_handle_t */
-            for (i = count - 1; i >= 0; i--)
-                list[i] = wine_server_ptr_handle( ((user_handle_t *)list)[i] );
-            list[count] = 0;
-            return list;
-        }
+        if (!(list = HeapAlloc( GetProcessHeap(), 0, size * sizeof(HWND) ))) return FALSE;
+        status = NtUserBuildHwndList( desktop, hwnd, children, TRUE, tid, size, list, &size );
+        if (!status) break;
         HeapFree( GetProcessHeap(), 0, list );
-        if (!count) break;
-        size = count + 1;  /* restart with a large enough buffer */
+        if (status != STATUS_BUFFER_TOO_SMALL) return FALSE;
     }
-    return NULL;
+    for (i = 0; i < size && list[i] != HWND_BOTTOM; i++)
+    {
+        if (!IsWindow( list[i] )) continue;
+        if (!(ret = enum_callback_wrapper( proc, list[i], param ))) break;
+    }
+    HeapFree( GetProcessHeap(), 0, list );
+    return ret;
 }
 
 
@@ -581,15 +588,6 @@ HWND WINAPI GetDesktopWindow(void)
 }
 
 
-/*******************************************************************
- *		EnableWindow (USER32.@)
- */
-BOOL WINAPI EnableWindow( HWND hwnd, BOOL enable )
-{
-    return NtUserEnableWindow( hwnd, enable );
-}
-
-
 /***********************************************************************
  *		IsWindowEnabled (USER32.@)
  */
@@ -615,6 +613,13 @@ DPI_AWARENESS_CONTEXT WINAPI GetWindowDpiAwarenessContext( HWND hwnd )
     return LongToHandle( NtUserGetWindowDpiAwarenessContext( hwnd ) );
 }
 
+/***********************************************************************
+ *		GetDpiAwarenessContextForProcess  (USER32.@)
+ */
+DPI_AWARENESS_CONTEXT WINAPI GetDpiAwarenessContextForProcess(HANDLE process)
+{
+    return LongToHandle( NtUserGetProcessDpiAwarenessContext( process ) );
+}
 
 /***********************************************************************
  *		GetWindowDpiHostingBehavior  (USER32.@)
@@ -673,7 +678,8 @@ void WINAPI SwitchToThisWindow( HWND hwnd, BOOL alt_tab )
  */
 BOOL WINAPI GetWindowRect( HWND hwnd, RECT *rect )
 {
-    BOOL ret = NtUserGetWindowRect( hwnd, rect );
+    UINT dpi = NTUSER_DPI_CONTEXT_GET_DPI( (UINT_PTR)GetThreadDpiAwarenessContext() );
+    BOOL ret = NtUserGetWindowRect( hwnd, rect, dpi );
     if (ret) TRACE( "hwnd %p %s\n", hwnd, wine_dbgstr_rect(rect) );
     return ret;
 }
@@ -715,7 +721,8 @@ int WINAPI GetWindowRgnBox( HWND hwnd, RECT *rect )
  */
 BOOL WINAPI GetClientRect( HWND hwnd, RECT *rect )
 {
-    return NtUserGetClientRect( hwnd, rect );
+    UINT dpi = NTUSER_DPI_CONTEXT_GET_DPI( (UINT_PTR)GetThreadDpiAwarenessContext() );
+    return NtUserGetClientRect( hwnd, rect, dpi );
 }
 
 
@@ -758,7 +765,8 @@ HWND WINAPI ChildWindowFromPointEx( HWND parent, POINT pt, UINT flags )
  */
 INT WINAPI MapWindowPoints( HWND hwnd_from, HWND hwnd_to, POINT *points, UINT count )
 {
-    return NtUserMapWindowPoints( hwnd_from, hwnd_to, points, count );
+    UINT dpi = NTUSER_DPI_CONTEXT_GET_DPI( (UINT_PTR)GetThreadDpiAwarenessContext() );
+    return NtUserMapWindowPoints( hwnd_from, hwnd_to, points, count, dpi );
 }
 
 
@@ -877,15 +885,6 @@ BOOL WINAPI EndDeferWindowPos( HDWP hdwp )
 }
 
 
-/***********************************************************************
- *           ArrangeIconicWindows (USER32.@)
- */
-UINT WINAPI ArrangeIconicWindows( HWND parent )
-{
-    return NtUserArrangeIconicWindows( parent );
-}
-
-
 /**********************************************************************
  *		GetWindowWord (USER32.@)
  */
@@ -1283,29 +1282,12 @@ HWND WINAPI GetWindow( HWND hwnd, UINT rel )
 }
 
 
-/*******************************************************************
- *		ShowOwnedPopups (USER32.@)
- */
-BOOL WINAPI ShowOwnedPopups( HWND owner, BOOL show )
-{
-    return NtUserShowOwnedPopups( owner, show );
-}
-
-
 /*******************************************************************
  *		GetLastActivePopup (USER32.@)
  */
 HWND WINAPI GetLastActivePopup( HWND hwnd )
 {
-    HWND retval = hwnd;
-
-    SERVER_START_REQ( get_window_info )
-    {
-        req->handle = wine_server_user_handle( hwnd );
-        if (!wine_server_call_err( req )) retval = wine_server_ptr_handle( reply->last_active );
-    }
-    SERVER_END_REQ;
-    return retval;
+    return NtUserGetLastActivePopup( hwnd );
 }
 
 
@@ -1317,12 +1299,22 @@ HWND WINAPI GetLastActivePopup( HWND hwnd )
  */
 HWND *WIN_ListChildren( HWND hwnd )
 {
-    if (!hwnd)
+    HWND *list;
+    ULONG size = 128;
+    NTSTATUS status;
+
+    if (!(hwnd = GetWindow( hwnd, GW_CHILD ))) return NULL;
+
+    for (;;)
     {
-        SetLastError( ERROR_INVALID_WINDOW_HANDLE );
-        return NULL;
+        if (!(list = HeapAlloc( GetProcessHeap(), 0, size * sizeof(HWND) ))) return NULL;
+        status = NtUserBuildHwndList( 0, hwnd, FALSE, TRUE, 0, size, list, &size );
+        if (!status && size > 1) break;
+        HeapFree( GetProcessHeap(), 0, list );
+        if (status != STATUS_BUFFER_TOO_SMALL) return NULL;
     }
-    return list_window_children( 0, hwnd, NULL, 0 );
+    list[size - 1] = 0;
+    return list;
 }
 
 
@@ -1331,26 +1323,7 @@ HWND *WIN_ListChildren( HWND hwnd )
  */
 BOOL WINAPI EnumWindows( WNDENUMPROC lpEnumFunc, LPARAM lParam )
 {
-    HWND *list;
-    BOOL ret = TRUE;
-    int i;
-
-    /* We have to build a list of all windows first, to avoid */
-    /* unpleasant side-effects, for instance if the callback */
-    /* function changes the Z-order of the windows.          */
-
-    if (!(list = WIN_ListChildren( GetDesktopWindow() ))) return TRUE;
-
-    /* Now call the callback function for every window */
-
-    for (i = 0; list[i]; i++)
-    {
-        /* Make sure that the window still exists */
-        if (!IsWindow( list[i] )) continue;
-        if (!(ret = lpEnumFunc( list[i], lParam ))) break;
-    }
-    HeapFree( GetProcessHeap(), 0, list );
-    return ret;
+    return enum_windows( 0, 0, 0, FALSE, lpEnumFunc, lParam );
 }
 
 
@@ -1359,18 +1332,7 @@ BOOL WINAPI EnumWindows( WNDENUMPROC lpEnumFunc, LPARAM lParam )
  */
 BOOL WINAPI EnumThreadWindows( DWORD id, WNDENUMPROC func, LPARAM lParam )
 {
-    HWND *list;
-    int i;
-    BOOL ret = TRUE;
-
-    if (!(list = list_window_children( 0, GetDesktopWindow(), NULL, id ))) return TRUE;
-
-    /* Now call the callback function for every window */
-
-    for (i = 0; list[i]; i++)
-        if (!(ret = func( list[i], lParam ))) break;
-    HeapFree( GetProcessHeap(), 0, list );
-    return ret;
+    return enum_windows( 0, 0, id, FALSE, func, lParam );
 }
 
 
@@ -1379,70 +1341,7 @@ BOOL WINAPI EnumThreadWindows( DWORD id, WNDENUMPROC func, LPARAM lParam )
  */
 BOOL WINAPI EnumDesktopWindows( HDESK desktop, WNDENUMPROC func, LPARAM lparam )
 {
-    HWND *list;
-    int i;
-
-    if (!(list = list_window_children( desktop, 0, NULL, 0 ))) return TRUE;
-
-    for (i = 0; list[i]; i++)
-        if (!func( list[i], lparam )) break;
-    HeapFree( GetProcessHeap(), 0, list );
-    return TRUE;
-}
-
-
-#ifdef __i386__
-/* Some apps pass a non-stdcall proc to EnumChildWindows,
- * so we need a small assembly wrapper to call the proc.
- */
-extern LRESULT enum_callback_wrapper( WNDENUMPROC proc, HWND hwnd, LPARAM lparam );
-__ASM_GLOBAL_FUNC( enum_callback_wrapper,
-    "pushl %ebp\n\t"
-    __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")
-    __ASM_CFI(".cfi_rel_offset %ebp,0\n\t")
-    "movl %esp,%ebp\n\t"
-    __ASM_CFI(".cfi_def_cfa_register %ebp\n\t")
-    "pushl 16(%ebp)\n\t"
-    "pushl 12(%ebp)\n\t"
-    "call *8(%ebp)\n\t"
-    "leave\n\t"
-    __ASM_CFI(".cfi_def_cfa %esp,4\n\t")
-    __ASM_CFI(".cfi_same_value %ebp\n\t")
-    "ret" )
-#else
-static inline LRESULT enum_callback_wrapper( WNDENUMPROC proc, HWND hwnd, LPARAM lparam )
-{
-    return proc( hwnd, lparam );
-}
-#endif /* __i386__ */
-
-/**********************************************************************
- *           WIN_EnumChildWindows
- *
- * Helper function for EnumChildWindows().
- */
-static BOOL WIN_EnumChildWindows( HWND *list, WNDENUMPROC func, LPARAM lParam )
-{
-    HWND *childList;
-    BOOL ret = FALSE;
-
-    for ( ; *list; list++)
-    {
-        /* Make sure that the window still exists */
-        if (!IsWindow( *list )) continue;
-        /* Build children list first */
-        childList = WIN_ListChildren( *list );
-
-        ret = enum_callback_wrapper( func, *list, lParam );
-
-        if (childList)
-        {
-            if (ret) ret = WIN_EnumChildWindows( childList, func, lParam );
-            HeapFree( GetProcessHeap(), 0, childList );
-        }
-        if (!ret) return FALSE;
-    }
-    return TRUE;
+    return enum_windows( desktop, 0, 0, FALSE, func, lparam );
 }
 
 
@@ -1451,13 +1350,7 @@ static BOOL WIN_EnumChildWindows( HWND *list, WNDENUMPROC func, LPARAM lParam )
  */
 BOOL WINAPI EnumChildWindows( HWND parent, WNDENUMPROC func, LPARAM lParam )
 {
-    HWND *list;
-    BOOL ret;
-
-    if (!(list = WIN_ListChildren( parent ))) return FALSE;
-    ret = WIN_EnumChildWindows( list, func, lParam );
-    HeapFree( GetProcessHeap(), 0, list );
-    return ret;
+    return enum_windows( 0, parent, 0, TRUE, func, lParam );
 }
 
 
@@ -1497,24 +1390,6 @@ BOOL WINAPI FlashWindow( HWND hWnd, BOOL bInvert )
 }
 
 
-/*******************************************************************
- *		GetWindowContextHelpId (USER32.@)
- */
-DWORD WINAPI GetWindowContextHelpId( HWND hwnd )
-{
-    return NtUserGetWindowContextHelpId( hwnd );
-}
-
-
-/*******************************************************************
- *		SetWindowContextHelpId (USER32.@)
- */
-BOOL WINAPI SetWindowContextHelpId( HWND hwnd, DWORD id )
-{
-    return NtUserSetWindowContextHelpId( hwnd, id );
-}
-
-
 /*******************************************************************
  *		DragDetect (USER32.@)
  */
@@ -1665,17 +1540,6 @@ BOOL WINAPI GetProcessDefaultLayout( DWORD *layout )
 }
 
 
-/******************************************************************************
- *                    SetProcessDefaultLayout [USER32.@]
- *
- * Sets the default layout for parentless windows.
- */
-BOOL WINAPI SetProcessDefaultLayout( DWORD layout )
-{
-    return NtUserSetProcessDefaultLayout( layout );
-}
-
-
 /***********************************************************************
  *           UpdateWindow (USER32.@)
  */
@@ -1691,21 +1555,6 @@ BOOL WINAPI UpdateWindow( HWND hwnd )
 }
 
 
-/***********************************************************************
- *           ValidateRgn (USER32.@)
- */
-BOOL WINAPI ValidateRgn( HWND hwnd, HRGN hrgn )
-{
-    if (!hwnd)
-    {
-        SetLastError( ERROR_INVALID_WINDOW_HANDLE );
-        return FALSE;
-    }
-
-    return NtUserRedrawWindow( hwnd, NULL, hrgn, RDW_VALIDATE );
-}
-
-
 /*************************************************************************
  *           ScrollWindow (USER32.@)
  */
