@@ -1146,19 +1146,16 @@ static void make_dc_dirty( struct dce *dce )
  * rectangle. In addition, pWnd->parent DCEs may need to be updated if
  * DCX_CLIPCHILDREN flag is set.
  */
-void invalidate_dce( WND *win, const RECT *extra_rect )
+void invalidate_dce( WND *win, const RECT *old_rect )
 {
     UINT context;
-    RECT window_rect;
     struct dce *dce;
 
     if (!win->parent) return;
 
     context = set_thread_dpi_awareness_context( get_window_dpi_awareness_context( win->obj.handle ));
-    get_window_rect( win->obj.handle, &window_rect, get_thread_dpi() );
 
-    TRACE("%p parent %p %s (%s)\n",
-          win->obj.handle, win->parent, wine_dbgstr_rect(&window_rect), wine_dbgstr_rect(extra_rect) );
+    TRACE("%p parent %p, old_rect %s\n", win->obj.handle, win->parent, wine_dbgstr_rect(old_rect) );
 
     /* walk all DCEs and fixup non-empty entries */
 
@@ -1182,11 +1179,26 @@ void invalidate_dce( WND *win, const RECT *extra_rect )
         /* otherwise check if the window rectangle intersects this DCE window */
         if (win->parent == dce->hwnd || is_child( win->parent, dce->hwnd ))
         {
-            RECT dce_rect, tmp;
-            get_window_rect( dce->hwnd, &dce_rect, get_thread_dpi() );
-            if (intersect_rect( &tmp, &dce_rect, &window_rect ) ||
-                (extra_rect && intersect_rect( &tmp, &dce_rect, extra_rect )))
-                make_dc_dirty( dce );
+            RECT tmp, new_window_rect, old_window_rect;
+            struct window_rects rects;
+
+            /* get the parent client-relative old/new window rects */
+            get_window_rects( win->obj.handle, COORDS_PARENT, &rects, get_thread_dpi() );
+            old_window_rect = old_rect ? *old_rect : rects.window;
+            new_window_rect = rects.window;
+
+            /* get the DCE window rect in client-relative coordinates */
+            get_window_rects( dce->hwnd, COORDS_CLIENT, &rects, get_thread_dpi() );
+            if (win->parent != dce->hwnd)
+            {
+                /* map the window rects from parent client-relative to DCE window client-relative coordinates */
+                map_window_points( win->parent, dce->hwnd, (POINT *)&new_window_rect, 2, get_thread_dpi() );
+                map_window_points( win->parent, dce->hwnd, (POINT *)&old_window_rect, 2, get_thread_dpi() );
+            }
+
+            /* check if any of the window rects intersects with the DCE window rect */
+            if (intersect_rect( &tmp, &rects.window, &old_window_rect )) make_dc_dirty( dce );
+            else if (intersect_rect( &tmp, &rects.window, &new_window_rect )) make_dc_dirty( dce );
         }
     }
     set_thread_dpi_awareness_context( context );
@@ -1612,22 +1624,20 @@ static BOOL send_erase( HWND hwnd, UINT flags, HRGN client_rgn,
  *
  * Move the window bits when a window is resized, or moved within a parent window.
  */
-void move_window_bits( HWND hwnd, const RECT *visible_rect, const RECT *old_visible_rect,
-                       const RECT *window_rect, const RECT *valid_rects )
+void move_window_bits( HWND hwnd, const struct window_rects *rects, const RECT *valid_rects )
 {
-    RECT dst = valid_rects[0];
-    RECT src = valid_rects[1];
+    RECT dst = valid_rects[0], src = valid_rects[1];
 
-    if (src.left - old_visible_rect->left != dst.left - visible_rect->left ||
-        src.top - old_visible_rect->top != dst.top - visible_rect->top)
+    if (src.left - rects->visible.left != dst.left - rects->visible.left ||
+        src.top - rects->visible.top != dst.top - rects->visible.top)
     {
         UINT flags = UPDATE_NOCHILDREN | UPDATE_CLIPCHILDREN;
         HRGN rgn = get_update_region( hwnd, &flags, NULL );
         HDC hdc = NtUserGetDCEx( hwnd, rgn, DCX_CACHE | DCX_WINDOW | DCX_EXCLUDERGN );
 
         TRACE( "copying %s -> %s\n", wine_dbgstr_rect( &src ), wine_dbgstr_rect( &dst ));
-        OffsetRect( &src, -window_rect->left, -window_rect->top );
-        OffsetRect( &dst, -window_rect->left, -window_rect->top );
+        OffsetRect( &src, -rects->window.left, -rects->window.top );
+        OffsetRect( &dst, -rects->window.left, -rects->window.top );
 
         NtGdiStretchBlt( hdc, dst.left, dst.top, dst.right - dst.left, dst.bottom - dst.top,
                          hdc, src.left, src.top, src.right - src.left, src.bottom - src.top, SRCCOPY, 0 );
@@ -1845,6 +1855,20 @@ BOOL WINAPI NtUserValidateRect( HWND hwnd, const RECT *rect )
     return NtUserRedrawWindow( hwnd, rect, 0, flags );
 }
 
+/***********************************************************************
+ *           NtUserValidateRgn (win32u.@)
+ */
+BOOL WINAPI NtUserValidateRgn( HWND hwnd, HRGN hrgn )
+{
+    if (!hwnd)
+    {
+        RtlSetLastWin32Error( ERROR_INVALID_WINDOW_HANDLE );
+        return FALSE;
+    }
+
+    return NtUserRedrawWindow( hwnd, NULL, hrgn, RDW_VALIDATE );
+}
+
 /***********************************************************************
  *           NtUserGetUpdateRgn (win32u.@)
  */
@@ -2145,7 +2169,7 @@ INT WINAPI NtUserScrollWindowEx( HWND hwnd, INT dx, INT dy, const RECT *rect,
 
     if (flags & SW_SCROLLCHILDREN)
     {
-        HWND *list = list_window_children( 0, hwnd, NULL, 0 );
+        HWND *list = list_window_children( hwnd );
         if (list)
         {
             RECT r, dummy;
@@ -2173,7 +2197,7 @@ INT WINAPI NtUserScrollWindowEx( HWND hwnd, INT dx, INT dy, const RECT *rect,
         NtGdiDeleteObjectApp( winupd_rgn );
     }
 
-    if (move_caret) set_caret_pos( new_caret_pos.x, new_caret_pos.y );
+    if (move_caret) NtUserSetCaretPos( new_caret_pos.x, new_caret_pos.y );
     if (caret_hwnd) NtUserShowCaret( caret_hwnd );
     if (own_rgn && update_rgn) NtGdiDeleteObjectApp( update_rgn );
 
