@@ -1107,6 +1107,11 @@ BOOL SYSCALL_API NtUserAddClipboardFormatListener( HWND hwnd )
     SYSCALL_FUNC( NtUserAddClipboardFormatListener );
 }
 
+UINT SYSCALL_API NtUserArrangeIconicWindows( HWND parent )
+{
+    SYSCALL_FUNC( NtUserArrangeIconicWindows );
+}
+
 UINT SYSCALL_API NtUserAssociateInputContext( HWND hwnd, HIMC ctx, ULONG flags )
 {
     SYSCALL_FUNC( NtUserAssociateInputContext );
@@ -1127,12 +1132,22 @@ NTSTATUS SYSCALL_API NtUserBuildHimcList( UINT thread_id, UINT count, HIMC *buff
     SYSCALL_FUNC( NtUserBuildHimcList );
 }
 
-NTSTATUS SYSCALL_API NtUserBuildHwndList( HDESK desktop, ULONG unk2, ULONG unk3, ULONG unk4,
+NTSTATUS SYSCALL_API NtUserBuildHwndList( HDESK desktop, HWND hwnd, BOOL children, BOOL non_immersive,
                                           ULONG thread_id, ULONG count, HWND *buffer, ULONG *size )
 {
     SYSCALL_FUNC( NtUserBuildHwndList );
 }
 
+NTSTATUS SYSCALL_API NtUserBuildNameList( HWINSTA winsta, ULONG size, struct ntuser_name_list *buffer, ULONG *ret_size )
+{
+    SYSCALL_FUNC( NtUserBuildNameList );
+}
+
+NTSTATUS SYSCALL_API NtUserBuildPropList( HWND hwnd, ULONG count, struct ntuser_property_list *buffer, ULONG *ret_count )
+{
+    SYSCALL_FUNC( NtUserBuildPropList );
+}
+
 ULONG_PTR SYSCALL_API NtUserCallHwnd( HWND hwnd, DWORD code )
 {
     SYSCALL_FUNC( NtUserCallHwnd );
@@ -1241,6 +1256,16 @@ HIMC SYSCALL_API NtUserCreateInputContext( UINT_PTR client_ptr )
     SYSCALL_FUNC( NtUserCreateInputContext );
 }
 
+HMENU SYSCALL_API NtUserCreateMenu(void)
+{
+    SYSCALL_FUNC( NtUserCreateMenu );
+}
+
+HMENU SYSCALL_API NtUserCreatePopupMenu(void)
+{
+    SYSCALL_FUNC( NtUserCreatePopupMenu );
+}
+
 HWND SYSCALL_API NtUserCreateWindowEx( DWORD ex_style, UNICODE_STRING *class_name,
                                        UNICODE_STRING *version, UNICODE_STRING *window_name,
                                        DWORD style, INT x, INT y, INT cx, INT cy,
@@ -1273,6 +1298,11 @@ BOOL SYSCALL_API NtUserDestroyAcceleratorTable( HACCEL handle )
     SYSCALL_FUNC( NtUserDestroyAcceleratorTable );
 }
 
+BOOL SYSCALL_API NtUserDestroyCaret(void)
+{
+    SYSCALL_FUNC( NtUserDestroyCaret );
+}
+
 BOOL SYSCALL_API NtUserDestroyCursor( HCURSOR cursor, ULONG arg )
 {
     SYSCALL_FUNC( NtUserDestroyCursor );
@@ -1330,6 +1360,11 @@ BOOL SYSCALL_API NtUserDrawIconEx( HDC hdc, INT x0, INT y0, HICON icon, INT widt
     SYSCALL_FUNC( NtUserDrawIconEx );
 }
 
+BOOL SYSCALL_API NtUserDrawMenuBar( HWND hwnd )
+{
+    SYSCALL_FUNC( NtUserDrawMenuBar );
+}
+
 DWORD SYSCALL_API NtUserDrawMenuBarTemp( HWND hwnd, HDC hdc, RECT *rect, HMENU handle, HFONT font )
 {
     SYSCALL_FUNC( NtUserDrawMenuBarTemp );
@@ -1350,11 +1385,21 @@ BOOL SYSCALL_API NtUserEnableMouseInPointer( BOOL enable )
     SYSCALL_FUNC( NtUserEnableMouseInPointer );
 }
 
+BOOL SYSCALL_API NtUserEnableMouseInPointerForThread(void)
+{
+    SYSCALL_FUNC( NtUserEnableMouseInPointerForThread );
+}
+
 BOOL SYSCALL_API NtUserEnableScrollBar( HWND hwnd, UINT bar, UINT flags )
 {
     SYSCALL_FUNC( NtUserEnableScrollBar );
 }
 
+BOOL SYSCALL_API NtUserEnableWindow( HWND hwnd, BOOL enable )
+{
+    SYSCALL_FUNC( NtUserEnableWindow );
+}
+
 BOOL SYSCALL_API NtUserEndDeferWindowPosEx( HDWP hdwp, BOOL async )
 {
     SYSCALL_FUNC( NtUserEndDeferWindowPosEx );
@@ -1370,6 +1415,11 @@ BOOL SYSCALL_API NtUserEndPaint( HWND hwnd, const PAINTSTRUCT *ps )
     SYSCALL_FUNC( NtUserEndPaint );
 }
 
+UINT SYSCALL_API NtUserEnumClipboardFormats( UINT format )
+{
+    SYSCALL_FUNC( NtUserEnumClipboardFormats );
+}
+
 NTSTATUS SYSCALL_API NtUserEnumDisplayDevices( UNICODE_STRING *device, DWORD index,
                                                DISPLAY_DEVICEW *info, DWORD flags )
 {
@@ -1443,6 +1493,11 @@ INT SYSCALL_API NtUserGetClassName( HWND hwnd, BOOL real, UNICODE_STRING *name )
     SYSCALL_FUNC( NtUserGetClassName );
 }
 
+BOOL SYSCALL_API NtUserGetClipCursor( RECT *rect )
+{
+    SYSCALL_FUNC( NtUserGetClipCursor );
+}
+
 HANDLE SYSCALL_API NtUserGetClipboardData( UINT format, struct get_clipboard_params *params )
 {
     SYSCALL_FUNC( NtUserGetClipboardData );
@@ -1468,6 +1523,11 @@ HWND SYSCALL_API NtUserGetClipboardViewer(void)
     SYSCALL_FUNC( NtUserGetClipboardViewer );
 }
 
+BOOL SYSCALL_API NtUserGetCurrentInputMessageSource( INPUT_MESSAGE_SOURCE *source )
+{
+    SYSCALL_FUNC( NtUserGetCurrentInputMessageSource );
+}
+
 HCURSOR SYSCALL_API NtUserGetCursor(void)
 {
     SYSCALL_FUNC( NtUserGetCursor );
@@ -1679,6 +1739,11 @@ HDESK SYSCALL_API NtUserGetThreadDesktop( DWORD thread )
     SYSCALL_FUNC( NtUserGetThreadDesktop );
 }
 
+ULONG_PTR SYSCALL_API NtUserGetThreadState( USERTHREADSTATECLASS cls )
+{
+    SYSCALL_FUNC( NtUserGetThreadState );
+}
+
 BOOL SYSCALL_API NtUserGetTitleBarInfo( HWND hwnd, TITLEBARINFO *info )
 {
     SYSCALL_FUNC( NtUserGetTitleBarInfo );
@@ -1699,6 +1764,11 @@ BOOL SYSCALL_API NtUserGetUpdatedClipboardFormats( UINT *formats, UINT size, UIN
     SYSCALL_FUNC( NtUserGetUpdatedClipboardFormats );
 }
 
+DWORD SYSCALL_API NtUserGetWindowContextHelpId( HWND hwnd )
+{
+    SYSCALL_FUNC( NtUserGetWindowContextHelpId );
+}
+
 HDC SYSCALL_API NtUserGetWindowDC( HWND hwnd )
 {
     SYSCALL_FUNC( NtUserGetWindowDC );
@@ -1724,9 +1794,9 @@ BOOL SYSCALL_API NtUserHiliteMenuItem( HWND hwnd, HMENU handle, UINT item, UINT
     SYSCALL_FUNC( NtUserHiliteMenuItem );
 }
 
-NTSTATUS SYSCALL_API NtUserInitializeClientPfnArrays( const struct user_client_procs *client_procsA,
-                                                      const struct user_client_procs *client_procsW,
-                                                      const void *client_workers, HINSTANCE user_module )
+NTSTATUS SYSCALL_API NtUserInitializeClientPfnArrays( const ntuser_client_func_ptr *client_procsA,
+                                                      const ntuser_client_func_ptr *client_procsW,
+                                                      const ntuser_client_func_ptr *client_workers, HINSTANCE user_module )
 {
     SYSCALL_FUNC( NtUserInitializeClientPfnArrays );
 }
@@ -1786,6 +1856,11 @@ INT SYSCALL_API NtUserMenuItemFromPoint( HWND hwnd, HMENU handle, int x, int y )
     SYSCALL_FUNC( NtUserMenuItemFromPoint );
 }
 
+BOOL SYSCALL_API NtUserMessageBeep( UINT type )
+{
+    SYSCALL_FUNC( NtUserMessageBeep );
+}
+
 LRESULT SYSCALL_API NtUserMessageCall( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam,
                                        void *result_info, DWORD type, BOOL ansi )
 {
@@ -1848,6 +1923,11 @@ BOOL SYSCALL_API NtUserPostMessage( HWND hwnd, UINT msg, WPARAM wparam, LPARAM l
     SYSCALL_FUNC( NtUserPostMessage );
 }
 
+BOOL SYSCALL_API NtUserPostQuitMessage( INT exit_code )
+{
+    SYSCALL_FUNC( NtUserPostQuitMessage );
+}
+
 BOOL SYSCALL_API NtUserPostThreadMessage( DWORD thread, UINT msg, WPARAM wparam, LPARAM lparam )
 {
     SYSCALL_FUNC( NtUserPostThreadMessage );
@@ -1870,11 +1950,21 @@ UINT_PTR SYSCALL_API NtUserQueryInputContext( HIMC handle, UINT attr )
     SYSCALL_FUNC( NtUserQueryInputContext );
 }
 
+HANDLE SYSCALL_API NtUserQueryWindow( HWND hwnd, WINDOWINFOCLASS cls )
+{
+    SYSCALL_FUNC( NtUserQueryWindow );
+}
+
 HWND SYSCALL_API NtUserRealChildWindowFromPoint( HWND parent, LONG x, LONG y )
 {
     SYSCALL_FUNC( NtUserRealChildWindowFromPoint );
 }
 
+UINT SYSCALL_API NtUserRealizePalette( HDC hdc )
+{
+    SYSCALL_FUNC( NtUserRealizePalette );
+}
+
 BOOL SYSCALL_API NtUserRedrawWindow( HWND hwnd, const RECT *rect, HRGN hrgn, UINT flags )
 {
     SYSCALL_FUNC( NtUserRedrawWindow );
@@ -1897,6 +1987,16 @@ BOOL SYSCALL_API NtUserRegisterRawInputDevices( const RAWINPUTDEVICE *devices, U
     SYSCALL_FUNC( NtUserRegisterRawInputDevices );
 }
 
+BOOL SYSCALL_API NtUserRegisterTouchPadCapable( BOOL capable )
+{
+    SYSCALL_FUNC( NtUserRegisterTouchPadCapable );
+}
+
+BOOL SYSCALL_API NtUserReleaseCapture(void)
+{
+    SYSCALL_FUNC( NtUserReleaseCapture );
+}
+
 INT SYSCALL_API NtUserReleaseDC( HWND hwnd, HDC hdc )
 {
     SYSCALL_FUNC( NtUserReleaseDC );
@@ -1917,6 +2017,11 @@ HANDLE SYSCALL_API NtUserRemoveProp( HWND hwnd, const WCHAR *str )
     SYSCALL_FUNC( NtUserRemoveProp );
 }
 
+BOOL SYSCALL_API NtUserReplyMessage( LRESULT result )
+{
+    SYSCALL_FUNC( NtUserReplyMessage );
+}
+
 BOOL SYSCALL_API NtUserScrollDC( HDC hdc, INT dx, INT dy, const RECT *scroll, const RECT *clip,
                                  HRGN ret_update_rgn, RECT *update_rect )
 {
@@ -1950,6 +2055,16 @@ HWND SYSCALL_API NtUserSetCapture( HWND hwnd )
     SYSCALL_FUNC( NtUserSetCapture );
 }
 
+BOOL SYSCALL_API NtUserSetCaretBlinkTime( unsigned int time )
+{
+    SYSCALL_FUNC( NtUserSetCaretBlinkTime );
+}
+
+BOOL SYSCALL_API NtUserSetCaretPos( INT x, INT y )
+{
+    SYSCALL_FUNC( NtUserSetCaretPos );
+}
+
 DWORD SYSCALL_API NtUserSetClassLong( HWND hwnd, INT offset, LONG newval, BOOL ansi )
 {
     SYSCALL_FUNC( NtUserSetClassLong );
@@ -2036,6 +2151,11 @@ HWND SYSCALL_API NtUserSetParent( HWND hwnd, HWND parent )
     SYSCALL_FUNC( NtUserSetParent );
 }
 
+BOOL SYSCALL_API NtUserSetProcessDefaultLayout( ULONG layout )
+{
+    SYSCALL_FUNC( NtUserSetProcessDefaultLayout );
+}
+
 BOOL SYSCALL_API NtUserSetProcessDpiAwarenessContext( ULONG awareness, ULONG unknown )
 {
     SYSCALL_FUNC( NtUserSetProcessDpiAwarenessContext );
@@ -2046,6 +2166,11 @@ BOOL SYSCALL_API NtUserSetProcessWindowStation( HWINSTA handle )
     SYSCALL_FUNC( NtUserSetProcessWindowStation );
 }
 
+HWND SYSCALL_API NtUserSetProgmanWindow( HWND hwnd )
+{
+    SYSCALL_FUNC( NtUserSetProgmanWindow );
+}
+
 BOOL SYSCALL_API NtUserSetProp( HWND hwnd, const WCHAR *str, HANDLE handle )
 {
     SYSCALL_FUNC( NtUserSetProp );
@@ -2076,6 +2201,11 @@ UINT_PTR SYSCALL_API NtUserSetSystemTimer( HWND hwnd, UINT_PTR id, UINT timeout
     SYSCALL_FUNC( NtUserSetSystemTimer );
 }
 
+HWND SYSCALL_API NtUserSetTaskmanWindow( HWND hwnd )
+{
+    SYSCALL_FUNC( NtUserSetTaskmanWindow );
+}
+
 BOOL SYSCALL_API NtUserSetThreadDesktop( HDESK handle )
 {
     SYSCALL_FUNC( NtUserSetThreadDesktop );
@@ -2093,6 +2223,11 @@ HWINEVENTHOOK SYSCALL_API NtUserSetWinEventHook( DWORD event_min, DWORD event_ma
     SYSCALL_FUNC( NtUserSetWinEventHook );
 }
 
+BOOL SYSCALL_API NtUserSetWindowContextHelpId( HWND hwnd, DWORD id )
+{
+    SYSCALL_FUNC( NtUserSetWindowContextHelpId );
+}
+
 LONG SYSCALL_API NtUserSetWindowLong( HWND hwnd, INT offset, LONG newval, BOOL ansi )
 {
     SYSCALL_FUNC( NtUserSetWindowLong );
@@ -2139,6 +2274,11 @@ INT SYSCALL_API NtUserShowCursor( BOOL show )
     SYSCALL_FUNC( NtUserShowCursor );
 }
 
+BOOL SYSCALL_API NtUserShowOwnedPopups( HWND owner, BOOL show )
+{
+    SYSCALL_FUNC( NtUserShowOwnedPopups );
+}
+
 BOOL SYSCALL_API NtUserShowScrollBar( HWND hwnd, INT bar, BOOL show )
 {
     SYSCALL_FUNC( NtUserShowScrollBar );
@@ -2212,6 +2352,11 @@ BOOL SYSCALL_API NtUserUnhookWinEvent( HWINEVENTHOOK handle )
     SYSCALL_FUNC( NtUserUnhookWinEvent );
 }
 
+BOOL SYSCALL_API NtUserUnhookWindowsHook( INT id, HOOKPROC proc )
+{
+    SYSCALL_FUNC( NtUserUnhookWindowsHook );
+}
+
 BOOL SYSCALL_API NtUserUnhookWindowsHookEx( HHOOK handle )
 {
     SYSCALL_FUNC( NtUserUnhookWindowsHookEx );
@@ -2245,6 +2390,11 @@ BOOL SYSCALL_API NtUserValidateRect( HWND hwnd, const RECT *rect )
     SYSCALL_FUNC( NtUserValidateRect );
 }
 
+BOOL SYSCALL_API NtUserValidateRgn( HWND hwnd, HRGN hrgn )
+{
+    SYSCALL_FUNC( NtUserValidateRgn );
+}
+
 WORD SYSCALL_API NtUserVkKeyScanEx( WCHAR chr, HKL layout )
 {
     SYSCALL_FUNC( NtUserVkKeyScanEx );
