@@ -37,6 +37,9 @@
 #define _WITH_CPU_SET_T
 #include <sched.h>
 #endif
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
@@ -61,7 +64,7 @@ struct thread_wait
     int                     count;      /* count of objects */
     int                     flags;
     int                     abandoned;
-    enum select_op          select;
+    enum select_opcode      select;
     client_ptr_t            key;        /* wait key for keyed events */
     client_ptr_t            cookie;     /* magic cookie to return to client */
     abstime_t               when;
@@ -79,8 +82,8 @@ struct thread_apc
     struct thread      *caller;   /* thread that queued this apc */
     struct object      *owner;    /* object that queued this apc */
     int                 executed; /* has it been executed by the client? */
-    apc_call_t          call;     /* call arguments */
-    apc_result_t        result;   /* call results once executed */
+    union apc_call      call;     /* call arguments */
+    union apc_result    result;   /* call results once executed */
 };
 
 static void dump_thread_apc( struct object *obj, int verbose );
@@ -119,7 +122,7 @@ struct context
 {
     struct object   obj;        /* object header */
     unsigned int    status;     /* status of the context */
-    context_t       regs[2];    /* context data */
+    struct context_data regs[2];/* context data */
 };
 #define CTX_NATIVE  0  /* context for native machine */
 #define CTX_WOW     1  /* context if thread is inside WoW */
@@ -216,6 +219,51 @@ static const struct fd_ops thread_fd_ops =
 
 static struct list thread_list = LIST_INIT(thread_list);
 
+#if defined(__linux__) && defined(RLIMIT_NICE)
+static int nice_limit;
+
+void init_threading(void)
+{
+    struct rlimit rlimit;
+
+    /* if wineserver has cap_sys_nice we are unlimited, but leave -20 to the user */
+    if (!setpriority( PRIO_PROCESS, getpid(), -20 )) nice_limit = -19;
+    setpriority( PRIO_PROCESS, getpid(), 0 );
+
+    if (!nice_limit && !getrlimit( RLIMIT_NICE, &rlimit ))
+    {
+        rlimit.rlim_cur = rlimit.rlim_max;
+        setrlimit( RLIMIT_NICE, &rlimit );
+        if (rlimit.rlim_max <= 40) nice_limit = 20 - rlimit.rlim_max;
+        else if (rlimit.rlim_max == -1) nice_limit = -20;
+        if (nice_limit >= 0 && debug_level) fprintf(stderr, "wine: RLIMIT_NICE is <= 20, unable to use setpriority safely\n");
+    }
+    if (nice_limit < 0 && debug_level) fprintf(stderr, "wine: Using setpriority to control niceness in the [%d,%d] range\n", nice_limit, -nice_limit );
+}
+
+static void apply_thread_priority( struct thread *thread, int base_priority )
+{
+    int min = -nice_limit, max = nice_limit, range = max - min, niceness;
+
+    /* FIXME: handle realtime priorities using SCHED_RR if possible */
+    if (base_priority > THREAD_BASE_PRIORITY_LOWRT) base_priority = THREAD_BASE_PRIORITY_LOWRT;
+    /* map an NT application band [1,15] base priority to [-nice_limit, nice_limit] */
+    niceness = (min + (base_priority - 1) * range / 14);
+    setpriority( PRIO_PROCESS, thread->unix_tid, niceness );
+}
+
+#else
+
+void init_threading(void)
+{
+}
+
+static void apply_thread_priority( struct thread *thread, int base_priority )
+{
+}
+
+#endif
+
 /* initialize the structure for a newly allocated thread */
 static inline void init_thread_structure( struct thread *thread )
 {
@@ -508,7 +556,7 @@ static void thread_apc_destroy( struct object *obj )
 }
 
 /* queue an async procedure call */
-static struct thread_apc *create_apc( struct object *owner, const apc_call_t *call_data )
+static struct thread_apc *create_apc( struct object *owner, const union apc_call *call_data )
 {
     struct thread_apc *apc;
 
@@ -605,28 +653,54 @@ affinity_t get_thread_affinity( struct thread *thread )
     return mask;
 }
 
+static int get_base_priority( int priority_class, int priority )
+{
+    /* offsets taken from https://learn.microsoft.com/en-us/windows/win32/procthread/scheduling-priorities */
+    static const int class_offsets[] = { 4, 8, 13, 24, 6, 10 };
+
+    if (priority == THREAD_PRIORITY_IDLE) return (priority_class == PROCESS_PRIOCLASS_REALTIME ? 16 : 1);
+    if (priority == THREAD_PRIORITY_TIME_CRITICAL) return (priority_class == PROCESS_PRIOCLASS_REALTIME ? 31 : 15);
+    if (priority_class >= ARRAY_SIZE(class_offsets)) return 8;
+    return class_offsets[priority_class - 1] + priority;
+}
+
 #define THREAD_PRIORITY_REALTIME_HIGHEST 6
 #define THREAD_PRIORITY_REALTIME_LOWEST -7
 
+unsigned int set_thread_priority( struct thread *thread, int priority_class, int priority )
+{
+    int max = THREAD_PRIORITY_HIGHEST;
+    int min = THREAD_PRIORITY_LOWEST;
+
+    if (priority_class == PROCESS_PRIOCLASS_REALTIME)
+    {
+        max = THREAD_PRIORITY_REALTIME_HIGHEST;
+        min = THREAD_PRIORITY_REALTIME_LOWEST;
+    }
+    if ((priority < min || priority > max) &&
+        priority != THREAD_PRIORITY_IDLE &&
+        priority != THREAD_PRIORITY_TIME_CRITICAL)
+        return STATUS_INVALID_PARAMETER;
+
+    if (thread->state == TERMINATED)
+        return STATUS_THREAD_IS_TERMINATING;
+
+    thread->priority = priority;
+
+    /* if thread is gone or hasn't started yet, this will be called again from init_thread with a unix_tid */
+    if (thread->unix_tid != -1) apply_thread_priority( thread, get_base_priority( priority_class, priority ));
+
+    return STATUS_SUCCESS;
+}
+
 /* set all information about a thread */
 static void set_thread_info( struct thread *thread,
                              const struct set_thread_info_request *req )
 {
     if (req->mask & SET_THREAD_INFO_PRIORITY)
     {
-        int max = THREAD_PRIORITY_HIGHEST;
-        int min = THREAD_PRIORITY_LOWEST;
-        if (thread->process->priority == PROCESS_PRIOCLASS_REALTIME)
-        {
-            max = THREAD_PRIORITY_REALTIME_HIGHEST;
-            min = THREAD_PRIORITY_REALTIME_LOWEST;
-        }
-        if ((req->priority >= min && req->priority <= max) ||
-            req->priority == THREAD_PRIORITY_IDLE ||
-            req->priority == THREAD_PRIORITY_TIME_CRITICAL)
-            thread->priority = req->priority;
-        else
-            set_error( STATUS_INVALID_PARAMETER );
+        unsigned int status = set_thread_priority( thread, thread->process->priority, req->priority );
+        if (status) set_error( status );
     }
     if (req->mask & SET_THREAD_INFO_AFFINITY)
     {
@@ -721,7 +795,7 @@ struct thread *get_wait_queue_thread( struct wait_queue_entry *entry )
     return entry->wait->thread;
 }
 
-enum select_op get_wait_queue_select_op( struct wait_queue_entry *entry )
+enum select_opcode get_wait_queue_select_op( struct wait_queue_entry *entry )
 {
     return entry->wait->select;
 }
@@ -775,7 +849,7 @@ static unsigned int end_wait( struct thread *thread, unsigned int status )
 }
 
 /* build the thread wait structure */
-static int wait_on( const select_op_t *select_op, unsigned int count, struct object *objects[],
+static int wait_on( const union select_op *select_op, unsigned int count, struct object *objects[],
                     int flags, abstime_t when )
 {
     struct thread_wait *wait;
@@ -812,7 +886,7 @@ static int wait_on( const select_op_t *select_op, unsigned int count, struct obj
     return current->wait ? 1 : 0;
 }
 
-static int wait_on_handles( const select_op_t *select_op, unsigned int count, const obj_handle_t *handles,
+static int wait_on_handles( const union select_op *select_op, unsigned int count, const obj_handle_t *handles,
                             int flags, abstime_t when )
 {
     struct object *objects[MAXIMUM_WAIT_OBJECTS];
@@ -981,7 +1055,7 @@ static int signal_object( obj_handle_t handle )
 }
 
 /* select on a list of handles */
-static int select_on( const select_op_t *select_op, data_size_t op_size, client_ptr_t cookie,
+static int select_on( const union select_op *select_op, data_size_t op_size, client_ptr_t cookie,
                       int flags, abstime_t when )
 {
     int ret;
@@ -996,8 +1070,8 @@ static int select_on( const select_op_t *select_op, data_size_t op_size, client_
 
     case SELECT_WAIT:
     case SELECT_WAIT_ALL:
-        count = (op_size - offsetof( select_op_t, wait.handles )) / sizeof(select_op->wait.handles[0]);
-        if (op_size < offsetof( select_op_t, wait.handles ) || count > MAXIMUM_WAIT_OBJECTS)
+        count = (op_size - offsetof( union select_op, wait.handles )) / sizeof(select_op->wait.handles[0]);
+        if (op_size < offsetof( union select_op, wait.handles ) || count > MAXIMUM_WAIT_OBJECTS)
         {
             set_error( STATUS_INVALID_PARAMETER );
             return 1;
@@ -1155,7 +1229,7 @@ static int queue_apc( struct process *process, struct thread *thread, struct thr
 }
 
 /* queue an async procedure call */
-int thread_queue_apc( struct process *process, struct thread *thread, struct object *owner, const apc_call_t *call_data )
+int thread_queue_apc( struct process *process, struct thread *thread, struct object *owner, const union apc_call *call_data )
 {
     struct thread_apc *apc;
     int ret = 0;
@@ -1299,7 +1373,7 @@ void kill_thread( struct thread *thread, int violent_death )
 }
 
 /* copy parts of a context structure */
-static void copy_context( context_t *to, const context_t *from, unsigned int flags )
+static void copy_context( struct context_data *to, const struct context_data *from, unsigned int flags )
 {
     assert( to->machine == from->machine );
     if (flags & SERVER_CTX_CONTROL) to->ctl = from->ctl;
@@ -1422,6 +1496,8 @@ DECL_HANDLER(init_first_thread)
     else
         set_thread_affinity( current, current->affinity );
 
+    set_thread_priority( current, process->priority, current->priority );
+
     debug_level = max( debug_level, req->debug_level );
 
     reply->pid          = get_process_id( process );
@@ -1451,6 +1527,7 @@ DECL_HANDLER(init_thread)
 
     init_thread_context( current );
     generate_debug_event( current, DbgCreateThreadStateChange, &req->entry );
+    set_thread_priority( current, current->process->priority, current->priority );
     set_thread_affinity( current, current->affinity );
 
     reply->suspend = (current->suspend || current->process->suspend || current->context != NULL);
@@ -1580,25 +1657,25 @@ DECL_HANDLER(resume_thread)
 /* select on a handle list */
 DECL_HANDLER(select)
 {
-    select_op_t select_op;
+    union select_op select_op;
     data_size_t op_size, ctx_size;
     struct context *ctx;
     struct thread_apc *apc;
-    const apc_result_t *result = get_req_data();
+    const union apc_result *result = get_req_data();
     unsigned int ctx_count;
 
     if (get_req_data_size() < sizeof(*result)) goto invalid_param;
     if (get_req_data_size() - sizeof(*result) < req->size) goto invalid_param;
     if (req->size & 3) goto invalid_param;
     ctx_size = get_req_data_size() - sizeof(*result) - req->size;
-    ctx_count = ctx_size / sizeof(context_t);
-    if (ctx_count * sizeof(context_t) != ctx_size) goto invalid_param;
+    ctx_count = ctx_size / sizeof(struct context_data);
+    if (ctx_count * sizeof(struct context_data) != ctx_size) goto invalid_param;
     if (ctx_count > 1 + (current->process->machine != native_machine)) goto invalid_param;
 
     if (ctx_count)
     {
-        const context_t *native_context = (const context_t *)((const char *)(result + 1) + req->size);
-        const context_t *wow_context = (ctx_count > 1) ? native_context + 1 : NULL;
+        const struct context_data *native_context = (const struct context_data *)((const char *)(result + 1) + req->size);
+        const struct context_data *wow_context = (ctx_count > 1) ? native_context + 1 : NULL;
 
         if (native_context->machine == native_machine)
         {
@@ -1665,13 +1742,13 @@ DECL_HANDLER(select)
 
     reply->signaled = select_on( &select_op, op_size, req->cookie, req->flags, req->timeout );
 
-    if (get_error() == STATUS_USER_APC && get_reply_max_size() >= sizeof(apc_call_t))
+    if (get_error() == STATUS_USER_APC && get_reply_max_size() >= sizeof(union apc_call))
     {
         apc = thread_dequeue_apc( current, 0 );
         set_reply_data( &apc->call, sizeof(apc->call) );
         release_object( apc );
     }
-    else if (get_error() == STATUS_KERNEL_APC && get_reply_max_size() >= sizeof(apc_call_t))
+    else if (get_error() == STATUS_KERNEL_APC && get_reply_max_size() >= sizeof(union apc_call))
     {
         apc = thread_dequeue_apc( current, 1 );
         if ((reply->apc_handle = alloc_handle( current->process, apc, SYNCHRONIZE, 0 )))
@@ -1685,14 +1762,14 @@ DECL_HANDLER(select)
         }
         release_object( apc );
     }
-    else if (reply->signaled && get_reply_max_size() >= sizeof(apc_call_t) + sizeof(context_t) &&
+    else if (reply->signaled && get_reply_max_size() >= sizeof(union apc_call) + sizeof(struct context_data) &&
              current->context && current->suspend_cookie == req->cookie)
     {
         ctx = current->context;
         if (ctx->regs[CTX_NATIVE].flags || ctx->regs[CTX_WOW].flags)
         {
-            apc_call_t *data;
-            data_size_t size = sizeof(*data) + (ctx->regs[CTX_WOW].flags ? 2 : 1) * sizeof(context_t);
+            union apc_call *data;
+            data_size_t size = sizeof(*data) + (ctx->regs[CTX_WOW].flags ? 2 : 1) * sizeof(struct context_data);
             unsigned int flags = system_flags & ctx->regs[CTX_NATIVE].flags;
 
             if (flags) set_thread_context( current, &ctx->regs[CTX_NATIVE], flags );
@@ -1718,7 +1795,7 @@ DECL_HANDLER(queue_apc)
     struct thread *thread = NULL;
     struct process *process = NULL;
     struct thread_apc *apc;
-    const apc_call_t *call = get_req_data();
+    const union apc_call *call = get_req_data();
 
     if (get_req_data_size() < sizeof(*call)) call = NULL;
 
@@ -1834,9 +1911,9 @@ DECL_HANDLER(get_thread_context)
 {
     struct context *thread_context = NULL;
     struct thread *thread;
-    context_t *context;
+    struct context_data *context;
 
-    if (get_reply_max_size() < 2 * sizeof(context_t))
+    if (get_reply_max_size() < 2 * sizeof(struct context_data))
     {
         set_error( STATUS_INVALID_PARAMETER );
         return;
@@ -1867,10 +1944,10 @@ DECL_HANDLER(get_thread_context)
                     get_thread_context( thread, &thread->context->regs[CTX_NATIVE], system_flags );
                 if (!get_error()) thread_context = (struct context *)grab_object( thread->context );
             }
-            else if (!get_error() && (context = set_reply_data_size( sizeof(context_t) )))
+            else if (!get_error() && (context = set_reply_data_size( sizeof(struct context_data) )))
             {
                 assert( reply->self );
-                memset( context, 0, sizeof(context_t) );
+                memset( context, 0, sizeof(struct context_data) );
                 context->machine = native_machine;
                 if (system_flags) get_thread_context( thread, context, system_flags );
             }
@@ -1888,7 +1965,7 @@ DECL_HANDLER(get_thread_context)
             native_flags = req->native_flags;
             wow_flags = req->flags & ~native_flags;
         }
-        if ((context = set_reply_data_size( (!!native_flags + !!wow_flags) * sizeof(context_t) )))
+        if ((context = set_reply_data_size( (!!native_flags + !!wow_flags) * sizeof(struct context_data) )))
         {
             if (native_flags)
             {
@@ -1921,10 +1998,10 @@ DECL_HANDLER(get_thread_context)
 DECL_HANDLER(set_thread_context)
 {
     struct thread *thread;
-    const context_t *contexts = get_req_data();
-    unsigned int ctx_count = get_req_data_size() / sizeof(context_t);
+    const struct context_data *contexts = get_req_data();
+    unsigned int ctx_count = get_req_data_size() / sizeof(struct context_data);
 
-    if (!ctx_count || ctx_count > 2 || ctx_count * sizeof(context_t) != get_req_data_size())
+    if (!ctx_count || ctx_count > 2 || ctx_count * sizeof(struct context_data) != get_req_data_size())
     {
         set_error( STATUS_INVALID_PARAMETER );
         return;
@@ -1952,7 +2029,7 @@ DECL_HANDLER(set_thread_context)
 
             if (ctx_count == 2 && (is_pending || thread->context->regs[CTX_WOW].machine))
             {
-                context_t *ctx = &thread->context->regs[CTX_WOW];
+                struct context_data *ctx = &thread->context->regs[CTX_WOW];
 
                 /* some regs are always set from the native context */
                 flags = contexts[CTX_WOW].flags & ~(req->native_flags | SERVER_CTX_EXEC_SPACE);
@@ -1965,7 +2042,7 @@ DECL_HANDLER(set_thread_context)
 
             if (native_flags)
             {
-                context_t *ctx = &thread->context->regs[CTX_NATIVE];
+                struct context_data *ctx = &thread->context->regs[CTX_NATIVE];
                 copy_context( ctx, &contexts[CTX_NATIVE], native_flags );
                 ctx->flags |= native_flags;
             }
@@ -2023,9 +2100,9 @@ DECL_HANDLER(get_next_thread)
     while (ptr)
     {
         thread = LIST_ENTRY( ptr, struct thread, entry );
-        if (thread->process == process)
+        if (thread->process == process &&
+            (reply->handle = alloc_handle( current->process, thread, req->access, req->attributes )))
         {
-            reply->handle = alloc_handle( current->process, thread, req->access, req->attributes );
             release_object( process );
             return;
         }
